/*
 * include/linux/gpio.h
 *
 * Copyright (C) 2004 Robert Schwebel, Pengutronix
 *		 2006 Ivan Korzakow
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifndef __GPIO_H
#define __GPIO_H

/* Values for policy */
#define GPIO_KERNEL	 (1<<0)
#define GPIO_USER	 (1<<1)
#define GPIO_INPUT	 (1<<2)
#define GPIO_OUTPUT	 (1<<3)

/** This event indicator is used to generate interrupts on high level */
#define GPIO_EVENT_HIGH_LEVEL	0
/** This event indicator is used to generate interrupts on low level */
#define GPIO_EVENT_LOW_LEVEL	1
/** This event indicator is used to generate interrupts on rising edge */
#define GPIO_EVENT_RISING_EDGE	2
/** This event indicator is used to generate interrupts on falling edge */
#define GPIO_EVENT_FALLING_EDGE	3
/** This event indicator is used to generate interrupts on both edges */
#define GPIO_EVENT_BOTH_EDGES	4

struct gpio_operations {
	int (* gpio_init_specific_op)(void);
	void (* gpio_free_resource_op)(void);
	int (* gpio_set_pin_op)(int gpio_nr);
	int (* gpio_clear_pin_op)(int gpio_nr);
	int (* gpio_get_pin_op)(int gpio_nr);
	int (* gpio_dir_input_op)(int gpio_nr);
	int (* gpio_dir_output_op)(int gpio_nr);
	int (* gpio_is_dir_input_op)(int gpio_nr);
	int (* gpio_enable_irq_op)(int gpio_nr, int event);
	int (* gpio_disable_irq_op)(int gpio_nr);
	void (* gpio_disable_all_irq_op)(void);
	unsigned int (* gpio_get_isr_op)(void);
	void (* gpio_ack_irq_op)(int gpio_nr);
};

struct gpio_params {
	struct gpio_operations *gpio_op;
	int gpio_pin_min;
	int gpio_pin_max;
	int irq_line;
};

int request_gpio(unsigned int pin_nr, const char *owner,
		 unsigned char policy, unsigned char init_level);
int free_gpio(unsigned int pin_nr);
int request_gpio_block(int first_pin, int last_pin, const char *owner,
		       unsigned char policy, unsigned int direction,
		       unsigned char init_level);
int free_gpio_block(int first_pin, int last_pin);
void free_all_gpios(void);


int gpio_request_irq(unsigned int gpio_nr, int trigger,
		     void (*handler)(int gpio_nr, void *private_data),
		     void *dev_id);
int gpio_free_irq(unsigned int gpio_nr);
int gpio_free_all_irq(void);


int register_gpio_specific(struct gpio_params *gpio_p);


int gpio_init(void);
int gpio_set_pin(int gpio_nr);
int gpio_clear_pin(int gpio_nr);
int gpio_dir_input(int gpio_nr);
int gpio_dir_output(int gpio_nr);
int gpio_get_pin(int gpio_nr);
int gpio_is_dir_input(int gpio_nr);


#endif
